/************************************************************/
/*															*/	
/*	This code computes										*/
/*	Radix-2 In Place Complex FFT on 64 to 2048 Points		*/
/*															*/
/*	Uses Decimation in Frequency Algorithm					*/
/*	Uses Look up tables for data unscrambling 				*/
/*															*/	
/*															*/	
/*	In_real : Input sequence in the DM						*/
/*			  Output would be stored in the same array		*/
/*			  Also the working array						*/
/*			  												*/	
/*	In_comp : Input sequence in the PM						*/
/*			  Output would be stored in the same array		*/
/*			  Also the working array						*/
/*			  												*/	
/*	Sin_twid:	Sin components of the twiddle factor		*/
/*				Would be stored in the PM					*/
/*															*/	
/*	Cos_twid:	Cos components of the twiddle factor		*/
/*				Would be stored in the DM					*/
/*															*/	
/*	lookup AND lookup: These Tables would be used for 		*/
/*				the unscrambling of the output				*/
/*															*/	
/*															*/	
/*			  												*/		  
/*	-- initial version 27th June, 2004 (Kunal Singh)		*/
/*															*/	
/*															*/	
/*	-- version: 9th July, 2004 (Kunal Singh)				*/	
/*			(i)	 Modified the butterfly loops for 			*/
/*				 Optimization.								*/
/*			(ii) Modified the Register Usage				*/
/*			(iii)Added Bench-Marks							*/
/*															*/	
/*															*/	
/*	-- version: 23rd July, 2004 (Kunal Singh)				*/	
/*			(i)	 Modified the middle stages butterfly 		*/
/*				 loop for Optimizations						*/
/*			(iii)Revised the Bench Marks					*/
/*															*/	
/*															*/
/*  -- version: 13th September, 2004 (Kunal Singh)			*/
/*			(i) Built the Demo Version with data, 			*/
/*				twid and look up table initialization 		*/
/*				from files									*/
/*															*/	
/*															*/	
/*															*/	
/************************************************************/



/********************************************/
/*											*/
/*			BENCH-MARKS						*/
/*											*/
/*	No of Points		Cycles				*/
/*											*/
/*		64				1438				*/
/*		128				2816				*/
/*		256				5873				*/
/*		512				12386				*/
/*		1024			26563				*/
/*		2048			56772				*/
/*											*/
/********************************************/

#include <Def21065l.h>


/************************************/
/*									*/
/*	DEFINE FFT SIZE 				*/
/*									*/
/************************************/
#define N 1024
#define N_2 N/2  	
#define N_4 N/4 	



/************************************/
/* 									*/
/*		LOOK UP TABLE SIZES			*/
/*									*/
/*   	N  	    M					*/
/*									*/
/*		64		28					*/
/*		128		56					*/
/*		256		120					*/
/*		512		240					*/
/*		1024	496					*/
/*		2048	992					*/
/*									*/
/************************************/
#define M 496


/****************************************************/
/* 													*/
/*				 REGISTER USAGE						*/
/*													*/
/*	R0, R1, R2, R3									*/
/*	R4, R5, R6, R7									*/
/*	R8, R9, R10, R12								*/
/*	R13, R14, R15, R16								*/
/*	Are used for the middle stage butterfly loop	*/ 
/*	None of the registers may be altered anywhere	*/
/*	outside the butterfly loop						*/
/*													*/	
/*													*/	
/*													*/
/*	The Following Registers SHOULD NOT BE ALTERED 	*/
/*	any where in the main FFT code 					*/
/*													*/
/*	ustat1 : Used to modify the data index			*/
/*	ustat2 : no_of_groups in a given stage;			*/
/*													*/
/****************************************************/



.section /dm seg_dmda;
/****************************************************/
/*													*/
/* Data Memory Section to store the Real Input 		*/
/* and the look up table							*/
/* The input is in 32-bit floating point format		*/
/* The look up table is in signed integer format	*/
/*													*/
/****************************************************/
.var In_real[N] = "test_data.dat"; 
.var lookup[M] = "mod.dat";


.section /pm seg_pmda;
/****************************************************/
/*													*/
/* Program Memory Data Section to store Imag Input 	*/
/* and the look up table							*/
/* The input is in 32-bit floating point format		*/
/* The look up table is in signed integer format	*/
/*													*/
/****************************************************/
.var In_comp[N];
.var rev_lookup[M] = "rev_mod.dat";



.section /dm seg_dm_twid;
/********************************************************/
/*														*/
/* Cos Twiddle factors - stored in the DM Data section	*/
/* This variable uses circular buffering 				*/
/* Address should be aligned to circ buffer boundary	*/
/*														*/
/********************************************************/
.var  Cos_twid[N/2] = "cos_twid.dat";


.section /pm seg_pm_twid;
/********************************************************/
/*														*/
/* Sin Twiddle factors - stored in the DM Data section	*/
/* Sign of the twiddle has been ignored 				*/
/* This variable uses circular buffering 				*/
/* Address should be aligned to circ buffer boundary	*/
/*														*/
/********************************************************/
.var Sin_twid[N/2] = "sin_twid.dat";



.section /pm seg_rth;
/************************************/
/* 									*/
/* INTERRUPT VECTOR TABLE			*/
/*									*/
/************************************/
nop; nop; nop; nop;
nop; jump start; nop; nop;	



.section /pm seg_pmco;
/************************************/
/* 									*/
/*		 MAIN PROGRAM CODE 			*/
/*									*/
/************************************/

start:

/********************************************/
/* 											*/
/*	Initialize the twiddle/lookup tables	*/
/*  before starting the actual FFT process  */ 
/*											*/
/********************************************/



	
		
	
/****************************************************/
/* 													*/
/*				  	INITIALIZATIONS 				*/
/*	 			For the DAG registers				*/
/*													*/
/* 													*/
/*	DAG register Usage:								*/
/* 													*/	
/*	I0:  To be used to fetch the operand x1			*/
/*	I1:  To be used to fetch the operand x2			*/
/*	I2:  To be used to fetch the cos_twid			*/
/* 													*/	
/*	I8:  To be used to fetch the operand y1			*/
/*	I9:  To be used to fetch the operand y2			*/
/*	I10: To be used to fetch the sin_twid			*/
/* 													*/	
/*	I3:  To be used to store the operand x1			*/
/*	I4:  To be used to store the operand x2			*/
/* 													*/	
/*	I11: To be used to store the operand y1			*/
/*	I12: To be used to store the operand y2			*/
/* 													*/	
/****************************************************/


	l0  = N;
	l1  = N;
	l3  = N;
	l4  = N;
	
	l8  = N;
	l9  = N;
	l11 = N;
	l12 = N;
	
	l2  = N_2;
	l10 = N_2;
	
	b0  = In_real;
	b1  = In_real;
	b3  = In_real;
	b4  = In_real;
	
	b8  = In_comp;
	b9  = In_comp;
	b11 = In_comp;
	b12 = In_comp;
	
	b2  = Cos_twid;
	b10 = Sin_twid;
	
	m1  = 1;
	m9  = 1;

	
	r11 = N_2;
				
	
	m0  = r11;
	m8  = r11;
	r0 = dm(i1,m0), r1 = pm(i9,m8);
	r0 = dm(i4,m0), r1 = pm(i12,m8);
	

/************************************************/
/*												*/
/*			First Stage Calculations			*/
/*												*/
/************************************************/
	
	r0 = r11 + 1;
	m4 = r0;
	m12 = r0;
	
	m2  = 1;
	m10 = 1;
	
/************************************/
/* 									*/
/*		Set up the Group Loop		*/
/*									*/
/************************************/	

	r8 = dm(i0,m1), r9 = pm(i8,m9);
	r12= dm(i1,m1), r13= pm(i9,m9);
	f2 = f8+f12, f0 = f8-f12, r4 = dm(i2,m2), r5 = pm(i10,m10);
	
	
	f10 = f0*f4, f3 = f9+f13, f1 = f9-f13;
	f14 = f1*f5, r8= dm(i0,m1), r9 = pm(i8,m9);
	f10 = f0*f5, f6 = f10+f14, r12 = dm(i1,m1), r13 = pm(i9,m9);
	f14 = f1*f4, f2 = f8+f12, f0 = f8-f12, dm(i3,m1) = r2, pm(i11,m9) = r3;
	f7 = f14-f10, r4 = dm(i2,m2), r5 = pm(i10,m10); 
	
	
/************************************/
/* 									*/
/*		Set up the But-fly Loop		*/
/*									*/
/************************************/		

	r8 = -3;
	r8 = r8+r11;
	lcntr = r8, do but_fly_loop_0 until lce;
	
	f10 = f0*f4, f3 = f9+f13, f1 = f9-f13, dm(i4,m1) = r6, pm(i12, m9) = r7;
	f14 = f1*f5, r8= dm(i0,m1), r9 = pm(i8,m9);
	f10 = f0*f5, f6 = f10+f14, r12 = dm(i1,m1), r13 = pm(i9,m9);
	f14 = f1*f4, f2 = f8+f12, f0 = f8-f12, dm(i3,m1) = r2, pm(i11,m9) = r3;
but_fly_loop_0: f7 = f14-f10, r4 = dm(i2,m2), r5 = pm(i10,m10); 	
	
/************************************/
/* 									*/
/*		End of the But-fly loop		*/
/*									*/
/************************************/		

	f10 = f0*f4, f3 = f9+f13, f1 = f9-f13, dm(i4,m1) = r6, pm(i12, m9) = r7;
	f14 = f1*f5, r8= dm(i0,m4), r9 = pm(i8,m12);
	f10 = f0*f5, f6 = f10+f14, r12 = dm(i1,m4), r13 = pm(i9,m12);
	f14 = f1*f4, f2 = f8+f12, f0 = f8-f12, dm(i3,m1) = r2, pm(i11,m9) = r3;
	f7 = f14-f10, r4 = dm(i2,m2), r5 = pm(i10,m10); 	
	
	f10 = f0*f4, f3 = f9+f13, f1 = f9-f13, dm(i4,m1) = r6, pm(i12, m9) = r7;
	f14 = f1*f5;
	f10 = f0*f5, f6 = f10+f14;
	f14 = f1*f4, dm(i3,m4) = r2, pm(i11,m12) = r3;
	f7 = f14-f10; 
	
	dm(i4,m4) = r6, pm(i12, m12) = r7;	

	 
/************************************/
/* 									*/
/*		End of the Group loop 		*/
/*									*/
/************************************/


	r0 = -r11;
	
	r11 =lshift r11 by -1;
	
	r0 = r0+r11;
	m3 = r0;
	m11 = r0;
	
	r0 = dm(i1,m3), r1 = pm(i9,m11);
	r0 = dm(i4,m3), r1 = pm(i12,m11);
	
/************************************************/
/*												*/
/*		End of the First Stage Calculations		*/
/*												*/
/************************************************/




/********************************************/
/*											*/
/*		SECOND STAGE CALCULATIONS			*/
/*											*/											
/********************************************/
	r0 = r11 + 1;
	m4 = r0;
	m12 = r0;
	
	m2  = 2;
	m10 = 2;
/************************************/
/* 									*/
/*		Set up the Group Loop		*/
/*									*/
/************************************/	
	lcntr = 2, do group_loop_1 until lce;
	
	

	r8 = dm(i0,m1), r9 = pm(i8,m9);
	r12= dm(i1,m1), r13= pm(i9,m9);
	f2 = f8+f12, f0 = f8-f12, r4 = dm(i2,m2), r5 = pm(i10,m10);
	
	
	f10 = f0*f4, f3 = f9+f13, f1 = f9-f13;
	f14 = f1*f5, r8= dm(i0,m1), r9 = pm(i8,m9);
	f10 = f0*f5, f6 = f10+f14, r12 = dm(i1,m1), r13 = pm(i9,m9);
	f14 = f1*f4, f2 = f8+f12, f0 = f8-f12, dm(i3,m1) = r2, pm(i11,m9) = r3;
	f7 = f14-f10, r4 = dm(i2,m2), r5 = pm(i10,m10); 
	
	
/************************************/
/* 									*/
/*		Set up the But-fly Loop		*/
/*									*/
/************************************/		
	r8 = -3;
	r8 = r8+r11;
	
	lcntr = r8, do but_fly_loop_1 until lce;
	
	f10 = f0*f4, f3 = f9+f13, f1 = f9-f13, dm(i4,m1) = r6, pm(i12, m9) = r7;
	f14 = f1*f5, r8= dm(i0,m1), r9 = pm(i8,m9);
	f10 = f0*f5, f6 = f10+f14, r12 = dm(i1,m1), r13 = pm(i9,m9);
	f14 = f1*f4, f2 = f8+f12, f0 = f8-f12, dm(i3,m1) = r2, pm(i11,m9) = r3;
but_fly_loop_1: f7 = f14-f10, r4 = dm(i2,m2), r5 = pm(i10,m10); 	
/************************************/
/* 									*/
/*		End of the But-fly loop		*/
/*									*/
/************************************/		

	f10 = f0*f4, f3 = f9+f13, f1 = f9-f13, dm(i4,m1) = r6, pm(i12, m9) = r7;
	f14 = f1*f5, r8= dm(i0,m4), r9 = pm(i8,m12);
	f10 = f0*f5, f6 = f10+f14, r12 = dm(i1,m4), r13 = pm(i9,m12);
	f14 = f1*f4, f2 = f8+f12, f0 = f8-f12, dm(i3,m1) = r2, pm(i11,m9) = r3;
	f7 = f14-f10, r4 = dm(i2,m2), r5 = pm(i10,m10); 	
	
	f10 = f0*f4, f3 = f9+f13, f1 = f9-f13, dm(i4,m1) = r6, pm(i12, m9) = r7;
	f14 = f1*f5;
	f10 = f0*f5, f6 = f10+f14;
	f14 = f1*f4, dm(i3,m4) = r2, pm(i11,m12) = r3;
	f7 = f14-f10; 
	
group_loop_1:	dm(i4,m4) = r6, pm(i12, m12) = r7;	
/************************************/
/* 									*/
/*		End of the Group loop 		*/
/*									*/
/************************************/

	r0 = -r11;
	
	r11 =lshift r11 by -1;
	
	r0 = r0+r11;
	m3 = r0;
	m11 = r0;
	
	r0 = dm(i1,m3), r1 = pm(i9,m11);
	r0 = dm(i4,m3), r1 = pm(i12,m11);
/********************************************/
/*											*/
/*	  END OF SECOND STAGE CALCULATIONS		*/											
/*											*/
/********************************************/





/********************************************/
/*											*/
/*		THIRD STAGE CALCULATIONS			*/
/*											*/											
/********************************************/

	r0 = r11 + 1;
	m4 = r0;
	m12 = r0;
	
	
	m2  = 4;
	m10 = 4;
	
/************************************/
/* 									*/
/*		Set up the Group Loop		*/
/*									*/
/************************************/	
	lcntr = 4, do group_loop_2 until lce;

	r8 = dm(i0,m1), r9 = pm(i8,m9);
	r12= dm(i1,m1), r13= pm(i9,m9);
	f2 = f8+f12, f0 = f8-f12, r4 = dm(i2,m2), r5 = pm(i10,m10);
	
	
	f10 = f0*f4, f3 = f9+f13, f1 = f9-f13;
	f14 = f1*f5, r8= dm(i0,m1), r9 = pm(i8,m9);
	f10 = f0*f5, f6 = f10+f14, r12 = dm(i1,m1), r13 = pm(i9,m9);
	f14 = f1*f4, f2 = f8+f12, f0 = f8-f12, dm(i3,m1) = r2, pm(i11,m9) = r3;
	f7 = f14-f10, r4 = dm(i2,m2), r5 = pm(i10,m10); 

/************************************/
/* 									*/
/*		Set up the But-fly Loop		*/
/*									*/
/************************************/		
	r8 = -3;
	r8 = r8+r11;
	
	lcntr = r8, do but_fly_loop_2 until lce;
	
	f10 = f0*f4, f3 = f9+f13, f1 = f9-f13, dm(i4,m1) = r6, pm(i12, m9) = r7;
	f14 = f1*f5, r8= dm(i0,m1), r9 = pm(i8,m9);
	f10 = f0*f5, f6 = f10+f14, r12 = dm(i1,m1), r13 = pm(i9,m9);
	f14 = f1*f4, f2 = f8+f12, f0 = f8-f12, dm(i3,m1) = r2, pm(i11,m9) = r3;
but_fly_loop_2: f7 = f14-f10, r4 = dm(i2,m2), r5 = pm(i10,m10); 	
/************************************/
/* 									*/
/*		End of the But-fly loop		*/
/*									*/
/************************************/		


	f10 = f0*f4, f3 = f9+f13, f1 = f9-f13, dm(i4,m1) = r6, pm(i12, m9) = r7;
	f14 = f1*f5, r8= dm(i0,m4), r9 = pm(i8,m12);
	f10 = f0*f5, f6 = f10+f14, r12 = dm(i1,m4), r13 = pm(i9,m12);
	f14 = f1*f4, f2 = f8+f12, f0 = f8-f12, dm(i3,m1) = r2, pm(i11,m9) = r3;
	f7 = f14-f10, r4 = dm(i2,m2), r5 = pm(i10,m10); 	
	
	f10 = f0*f4, f3 = f9+f13, f1 = f9-f13, dm(i4,m1) = r6, pm(i12, m9) = r7;
	f14 = f1*f5;
	f10 = f0*f5, f6 = f10+f14;
	f14 = f1*f4, dm(i3,m4) = r2, pm(i11,m12) = r3;
	f7 = f14-f10; 
	
group_loop_2:	dm(i4,m4) = r6, pm(i12, m12) = r7;	

			
	
	 
/************************************/
/* 									*/
/*		End of the Group loop 		*/
/*									*/
/************************************/
	r0 = -r11;
	
	r11 =lshift r11 by -1;
	
	r0 = r0+r11;
	m3 = r0;
	m11 = r0;
	
	r0 = dm(i1,m3), r1 = pm(i9,m11);
	r0 = dm(i4,m3), r1 = pm(i12,m11);
	

/********************************************/
/*											*/
/*	  END OF THIRD STAGE CALCULATIONS		*/											
/*											*/
/********************************************/




						
/****************************************************/
/* 													*/
/* 			R13:log2(N)								*/
/*													*/
/*	NO OF STAGES - LOG2(N)							*/
/*													*/
/* No need of the following calculations if the 	*/
/* R13 is directly initialized with the 			*/
/* above value. 									*/
/*													*/
/* This value of R13 would be used to set up the 	*/
/* stage loop - the R13 can be used as a GP reg. 	*/
/* once the stage loop has been set up.				*/
/*													*/	
/****************************************************/	

	r13 = -1;
	r2 = 1;
	r0 = N;
Cal_log:	r13 = r2+r13; 
			r0 = LSHIFT r0 by -1 ;
			r0 = pass r0;
			if ne jump Cal_log;
		
		
/********************************************/
/* 											*/
/*		Set up the Middle Stage Loop		*/
/*											*/
/********************************************/
		
		r15 = 8; 
	
		r0 = -5; 
		
		r13 = r13 + r0 ;
		lcntr = r13, do stage_loop until lce;
					
		
		ustat1 = r11;
		ustat2 = r15;						
				
				
		r0 = lshift r11 by 1;
		
		m0 = r0;
		m8 = r0;
		
		r0 = r0 + 1;
		m1 = r0;
		m9 = r0;

		m2 = ustat2;
		m10 = ustat2;	

		
		lcntr = ustat1, do outer_loop until lce;
/************************************/
/* 									*/
/*		Start of the Outer loop 	*/
/*									*/
/************************************/
		
		r8 = dm(i0,m0), r9 = pm(i8,m8);
		r12 = dm(i1,m0), r13 = pm(i9,m8);
		f3 = f8+f12, f1 = f8-f12, f4 = dm(i2,m2), f5 = pm(i10,m10);
		f10 = f1*f4, f7 = f9 + f13, f0 = f9-f13;
		f15 = f0*f5, r8 = dm(i0,m0), r9 = pm(i8,m8);
		f14 = f1*f5, f2 = f10+f15, r12 = dm(i1,m0), r13 = pm(i9,m8);
		f11 = f0*f4, f3 = f8+f12, f1 = f8-f12, dm(i3,m0) = r3, pm(i11, m8) = r7;
		f10 = f1*f4, f7 = f9 + f13, f0 = f9-f13, r8 = dm(i0,m0), r9 = pm(i8,m8);
		f14 = f1*f5, f6 = f11-f14,  r12 = dm(i1,m0), r13 = pm(i9,m8); 
		

		r1 = -4;
	
		r15 = ustat2;
		r15 = r15+r1;
		lcntr = r15, do inner_loop until lce;
/************************************/
/* 									*/
/*		Start of the Inner loop 	*/
/*									*/
/************************************/

		f11 = f0*f4, f3 = f8+f12, f1 = f8-f12, dm(i3,m0) = r3, pm(i11, m8) = r7;
		f15 = f0*f5, f7 = f9+f13, f0 = f9-f13, dm(i4,m0) = r2, pm(i12, m8) = r6;
		f10 = f1*f4, f2 = f10+f15, r8 = dm(i0,m0), r9 = pm(i8,m8); 
	
		inner_loop: f14 = f1*f5, f6 = f11-f14, r12 = dm(i1,m0), r13 = pm(i9,m8); 

/************************************/
/* 									*/
/*		End of the Outer loop 		*/
/*									*/
/************************************/
		
		
		
		f11 = f0*f4, f3 = f8+f12, f1 = f8-f12, dm(i3,m0) = r3, pm(i11, m8) = r7;
		f15 = f0*f5, f7 = f9+f13, f0 = f9-f13, dm(i4,m0) = r2, pm(i12, m8) = r6;
		f10 = f1*f4, f2 = f10+f15, r8 = dm(i0,m1), r9 = pm(i8,m9); 
		f14 = f1*f5, f6 = f11-f14, r12 = dm(i1,m1), r13 = pm(i9,m9); 

		f11 = f0*f4, f3 = f8+f12, f1 = f8-f12, dm(i3,m0) = r3, pm(i11, m8) = r7;
		f15 = f0*f5, f7 = f9+f13, f0 = f9-f13, dm(i4,m0) = r2, pm(i12, m8) = r6;
		f10 = f1*f4, f2 = f10+f15;
		f15 = f0*f5, f6 = f11-f14;
						
		f11 = f0*f4, f2 = f10+f15, dm(i4,m0) = r2, pm(i12, m8) = r6; 
		f14 = f1*f5,  dm(i3,m1) = r3, pm(i11, m9) = r7;
		f6 = f11-f14; 

		outer_loop: dm(i4,m1) = r2, pm(i12, m9) = r6;

/************************************/
/* 									*/
/*		End of the Outer loop 		*/
/*									*/
/************************************/

	r11 = ustat1;
	r15 = ustat2;
	
	r0 = -r11;
	m0 = r0;
	m8 = r0;
		
	r11 =lshift r11 by -1;
	
	r1 = -r11;
	r1 = r1+r0;
	m1 = r1;
	m9 = r1;
	
	 
	r12 = dm(i0,m0), r13 = pm(i8,m8); 
	r12 = dm(i3,m0), r13 = pm(i11,m8); 
	
	r12 = dm(i1,m1), r13 = pm(i9,m9); 
	r12 = dm(i4,m1), r13 = pm(i12,m9);
	
	
stage_loop:	r15 =lshift r15 by 1;


/********************************************/
/* 											*/
/*		End of the middle stages loop 		*/
/*											*/
/********************************************/
	





/****************************************************/
/* 													*/
/*		Last two stages for the FFT					*/
/*  Has been implemented with 4-point butterflies	*/
/*													*/
/****************************************************/
		
		i0 = In_real;
		i8 = In_comp;
		m6 = -7;
		m14 = -7;
		m7 = -1;
		m15 = -1;
		m5 = 2;
		m13 = 2;
		m2 = 5;
		m10 = 5;
		
		m0 = 4;
		m8 = 4;
		
		m1 = 5;
		m9 = 5;
		
		m3 = 1;
		m11 = 1;
		
		m4 = 3;
		m12 = 3;
		
		
		r0 = dm(i0,m5),		r8 = pm(i8,m13);
		r1 = dm(i0,m7), r9 = pm(i8,m15);
		f4 = f0+f1, f5 = f0-f1, r2 = dm(i0,m5), r10 = pm(i8,m13);
		f12 = f8+f9, f13 = f8-f9, r3 = dm(i0,m3), r11 = pm(i8,m11);
		f6 = f2+f3, f7 = f2-f3, r0 = dm(i0,m5),	r8 = pm(i8,m13);
		f14 = f10+f11, f15 = f10-f11, r1 = dm(i0,m7), r9 = pm(i8,m15);
		f4 = f4+f6, f6 = f4-f6, r2 = dm(i0,m5), r10 = pm(i8,m13);
		f12 = f12+f14, f14 = f12-f14, r3 = dm(i0,m6), r11 = pm(i8,m14);
		f5 = f5+f15, f15 = f5-f15, dm(i0,m3) = r4, pm(i8,m11)= r12;
		f13 = f13+f7, f7= f13-f7, dm(i0,m3) = r6, pm(i8,m11)= r14;

		
/************************************/
/* 									*/
/*	Set up the radix-4 FFT Loop		*/
/*									*/
/************************************/
r14 = N_4;
r4 = -1;
r14 = r14 + r4;
lcntr = r14, do last_two_stages until lce;
		
		
		f4 = f0+f1, f5 = f0-f1, dm(i0,m3) = r5, pm(i8,m11)= r7;
		f12 = f8+f9, f13 = f8-f9, dm(i0,m2) = r15, pm(i8,m10)= r13;
		f6 = f2+f3, f7 = f2-f3, r0 = dm(i0,m5),	r8 = pm(i8,m13);
		f14 = f10+f11, f15 = f10-f11, r1 = dm(i0,m7), r9 = pm(i8,m15);
		f4 = f4+f6, f6 = f4-f6, r2 = dm(i0,m5), r10 = pm(i8,m13);
		f12 = f12+f14, f14 = f12-f14, r3 = dm(i0,m6), r11 = pm(i8,m14);
		f5 = f5+f15, f15 = f5-f15, dm(i0,m3) = r4, pm(i8,m11)= r12;
	last_two_stages:	f13 = f13+f7, f7= f13-f7, dm(i0,m3) = r6, pm(i8,m11)= r14;
/************************************/
/* 									*/
/*	End of the radix-4 FFT Loop		*/
/*									*/
/************************************/
		
		
		dm(i0,m3) = r5, pm(i8,m11)= r7;
		dm(i0,m3) = r15, pm(i8,m11)= r13;
	
		

/****************************************/
/* 										*/
/* The FFT outputs are available in the */
/* In_real and In_comp Buffers			*/
/*										*/
/* The output needs to unscrambled 		*/
/*										*/
/****************************************/		
		

	i0 = In_real;	i8 = In_comp;
	i4 = In_real;	i12 = In_comp;
	i2 = lookup;	i10 = rev_lookup;
	
	l4 = 0;			l12 = 0;
	l2 = 0;			l10 = 0;
	l0 = 0;			l8 = 0;
	
	m2 = 1;			m10 = 1;
	m3 = 0;			m11 = 0;
	
	r5 = dm(i2,m2), r6 = pm(i10,m10);
	
	m0 = r5;	m8 = r5;
	m4 = r6;	m12 = r6;

	r0 = dm(i0,m0), r1 = pm(i8,m8);
	r2 = dm(i4,m4), r3 = pm(i12,m12);
	
	
/************************************/
/* 									*/
/* Set up the unscramble loop		*/
/*									*/
/************************************/
	
lcntr = M, do unscramble until lce;

	r5 = dm(i2,m2), r6 = pm(i10,m10);
	m0 = r5;	m8 = r5;
	m4 = r6;	m12 = r6;
	r0 = dm(i0,m3), r1 = pm(i8,m11);
	r2 = dm(i4,m3), r3 = pm(i12,m11);
	dm(i0,m0) = r2, pm(i8,m8) = r3;
 unscramble:	dm(i4,m4) = r0, pm(i12,m12) = r1;
	
/************************************/
/* 									*/
/* End of the unscramble loop		*/
/*									*/
/************************************/
	

	jump(pc,0);
	
	
	
